@@ -1,7 +1,7 @@
 /*
  * This file is part of GPaste.
  *
- * Copyright (c) 2010-2016, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
+ * Copyright (c) 2010-2018, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  */
 
 #include <gpaste-clipboard.h>
@@ -54,7 +54,12 @@ g_paste_clipboard_bootstrap_finish (GPasteClipboard *self,
     {
         const GList *h = g_paste_history_get_history (history);
         if (h)
-            g_paste_clipboard_select_item (self, h->data);
+        {
+            GPasteItem *item = h->data;
+
+            if (!g_paste_clipboard_select_item (self, item))
+                g_paste_history_remove (history, 0);
+        }
     }
 }
 
@@ -68,9 +73,10 @@ g_paste_clipboard_bootstrap_finish_text (GPasteClipboard *self,
 
 static void
 g_paste_clipboard_bootstrap_finish_image (GPasteClipboard *self,
-                                          GdkPixbuf       *image G_GNUC_UNUSED,
+                                          GdkPixbuf       *image,
                                           gpointer         user_data)
 {
+    g_object_unref (image);
     g_paste_clipboard_bootstrap_finish (self, user_data);
 }
 
@@ -111,21 +117,21 @@ g_paste_clipboard_bootstrap (GPasteClipboard *self,
 }
 
 /**
- * g_paste_clipboard_get_target:
+ * g_paste_clipboard_is_clipboard:
  * @self: a #GPasteClipboard instance
  *
- * Get the target the #GPasteClipboard points to
+ * Get whether this #GPasteClipboard is a clipboard or not (primary selection)
  *
- * Returns: (transfer none): the GdkAtom representing the target (Primary, Clipboard, ...)
+ * Returns: %TRUE if this #GPasteClipboard is a clipboard
  */
-G_PASTE_VISIBLE GdkAtom
-g_paste_clipboard_get_target (const GPasteClipboard *self)
+G_PASTE_VISIBLE gboolean
+g_paste_clipboard_is_clipboard (const GPasteClipboard *self)
 {
-    g_return_val_if_fail (_G_PASTE_IS_CLIPBOARD (self), NULL);
+    g_return_val_if_fail (_G_PASTE_IS_CLIPBOARD (self), FALSE);
 
     const GPasteClipboardPrivate *priv = _g_paste_clipboard_get_instance_private (self);
 
-    return gtk_clipboard_get_selection (priv->real);
+    return gtk_clipboard_get_selection (priv->real) == GDK_SELECTION_CLIPBOARD;
 }
 
 /**
@@ -164,6 +170,19 @@ g_paste_clipboard_get_text (const GPasteClipboard *self)
     return priv->text;
 }
 
+static const gchar *
+_g_paste_clipboard_private_target_name (const GPasteClipboardPrivate *priv)
+{
+    GdkAtom target = gtk_clipboard_get_selection (priv->real);
+
+    if (target == GDK_SELECTION_CLIPBOARD)
+        return "CLIPBOARD";
+    else if (target == GDK_SELECTION_PRIMARY)
+        return "PRIMARY";
+    else
+        return "UNKNOWN";
+}
+
 static void
 g_paste_clipboard_private_set_text (GPasteClipboardPrivate *priv,
                                     const gchar            *text)
@@ -171,7 +190,7 @@ g_paste_clipboard_private_set_text (GPasteClipboardPrivate *priv,
     g_free (priv->text);
     g_free (priv->image_checksum);
 
-    g_debug("%s: set text", gdk_atom_name (gtk_clipboard_get_selection (priv->real)));
+    g_debug("%s: set text", _g_paste_clipboard_private_target_name (priv));
 
     priv->text = g_strdup (text);
     priv->image_checksum = NULL;
@@ -275,13 +294,77 @@ g_paste_clipboard_select_text (GPasteClipboard *self,
 
     GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
 
-    g_debug("%s: select text", gdk_atom_name (gtk_clipboard_get_selection (priv->real)));
+    g_debug("%s: select text", _g_paste_clipboard_private_target_name (priv));
 
     /* Avoid cycling twice as gtk_clipboard_set_text will make the clipboards manager react */
     g_paste_clipboard_private_set_text (priv, text);
     gtk_clipboard_set_text (priv->real, text, -1);
 }
 
+static void
+g_paste_clipboard_sync_ready (GtkClipboard *clipboard G_GNUC_UNUSED,
+                              const gchar  *text,
+                              gpointer      user_data)
+{
+    if (text)
+        g_paste_clipboard_select_text (user_data, text);
+}
+
+/**
+ * g_paste_clipboard_sync_text:
+ * @self: the source #GPasteClipboard instance
+ * @other: the target #GPasteClipboard instance
+ *
+ * Synchronise the text between two clipboards
+ */
+G_PASTE_VISIBLE void
+g_paste_clipboard_sync_text (const GPasteClipboard *self,
+                             GPasteClipboard       *other)
+{
+    g_return_if_fail (_G_PASTE_IS_CLIPBOARD (self));
+    g_return_if_fail (_G_PASTE_IS_CLIPBOARD (other));
+
+    const GPasteClipboardPrivate *priv = _g_paste_clipboard_get_instance_private (self);
+
+    gtk_clipboard_request_text (priv->real, g_paste_clipboard_sync_ready, other);
+}
+
+static guchar *
+copy_str_as_uchars (const gchar *str,
+                    guint64      length)
+{
+    guchar *data = g_new (guchar, length);
+    for (guint64 i = 0; i < length; ++i)
+        data[i] = (guchar) str[i];
+    return data;
+}
+
+static void
+_get_clipboard_data_from_special_atom (GtkSelectionData *selection_data,
+                                       const GPasteItem *item,
+                                       GPasteSpecialAtom atom)
+{
+    if (atom >= G_PASTE_SPECIAL_ATOM_FIRST && atom < G_PASTE_SPECIAL_ATOM_LAST)
+    {
+        g_autofree guchar *data = NULL;
+        guint64 length = 0;
+        const gchar *str = g_paste_item_get_special_value (item, atom);
+
+        if (str)
+        {
+            data = g_base64_decode (str, &length);
+        }
+        else
+        {
+            str = g_paste_item_get_value (item);
+            length = strlen (str);
+            data = copy_str_as_uchars (str, length);
+        }
+
+        gtk_selection_data_set (selection_data, g_paste_special_atom_get (atom), 8, data, length);
+    }
+}
+
 static void
 g_paste_clipboard_get_clipboard_data (GtkClipboard     *clipboard G_GNUC_UNUSED,
                                       GtkSelectionData *selection_data,
@@ -292,42 +375,38 @@ g_paste_clipboard_get_clipboard_data (GtkClipboard     *clipboard G_GNUC_UNUSED,
 
     GPasteItem *item = G_PASTE_ITEM (user_data_or_owner);
 
-    GdkAtom targets[1] = { gtk_selection_data_get_target (selection_data) };
+    GdkAtom target = gtk_selection_data_get_target (selection_data);
+    GdkAtom targets[1] = { target };
+
+    if (_G_PASTE_IS_IMAGE_ITEM (item))
+    {
+        if (gtk_targets_include_image (targets, 1, TRUE))
+            gtk_selection_data_set_pixbuf (selection_data, g_paste_image_item_get_image (G_PASTE_IMAGE_ITEM (item)));
+        return;
+    }
+    else if (_G_PASTE_IS_URIS_ITEM (item))
+    {
+        if (gtk_targets_include_uri (targets, 1))
+        {
+            const gchar * const *uris = g_paste_uris_item_get_uris (G_PASTE_URIS_ITEM (item));
+
+            gtk_selection_data_set_uris (selection_data, (GStrv) uris);
+            return;
+        }
+    }
+
+    for (GPasteSpecialAtom a = G_PASTE_SPECIAL_ATOM_FIRST; a < G_PASTE_SPECIAL_ATOM_LAST; ++a)
+    {
+        if (target == g_paste_special_atom_get (a))
+        {
+            _get_clipboard_data_from_special_atom (selection_data, item, a);
+            return;
+        }
+    }
 
     /* The content is requested as text */
     if (gtk_targets_include_text (targets, 1))
         gtk_selection_data_set_text (selection_data, g_paste_item_get_real_value (item), -1);
-    else if (_G_PASTE_IS_IMAGE_ITEM (item))
-    {
-        if (gtk_targets_include_image (targets, 1, TRUE))
-            gtk_selection_data_set_pixbuf (selection_data, g_paste_image_item_get_image (G_PASTE_IMAGE_ITEM (item)));
-    }
-    /* The content is requested as uris */
-    else
-    {
-        g_return_if_fail (_G_PASTE_IS_URIS_ITEM (item));
-
-        const gchar * const *uris = g_paste_uris_item_get_uris (G_PASTE_URIS_ITEM (item));
-
-        if (gtk_targets_include_uri (targets, 1))
-            gtk_selection_data_set_uris (selection_data, (GStrv) uris);
-        /* The content is requested as special gnome-copied-files by nautilus */
-        else
-        {
-            g_autoptr (GString) copy_string = g_string_new ("copy");
-            guint64 length = g_strv_length ((GStrv) uris);
-
-            for (guint64 i = 0; i < length; ++i)
-                g_string_append_printf (copy_string, "\n%s", uris[i]);
-
-            gchar *str = copy_string->str;
-            length = copy_string->len + 1;
-            g_autofree guchar *copy_files_data = g_new (guchar, length);
-            for (guint64 i = 0; i < length; ++i)
-                copy_files_data[i] = (guchar) str[i];
-            gtk_selection_data_set (selection_data, g_paste_clipboard_copy_files_target, 8, copy_files_data, length);
-        }
-    }
 }
 
 static void
@@ -337,32 +416,6 @@ g_paste_clipboard_clear_clipboard_data (GtkClipboard *clipboard G_GNUC_UNUSED,
     g_object_unref (user_data_or_owner);
 }
 
-static void
-g_paste_clipboard_private_select_uris (GPasteClipboardPrivate *priv,
-                                       GPasteUrisItem         *item)
-{
-    GtkClipboard *real = priv->real;
-    g_autoptr (GtkTargetList) target_list = gtk_target_list_new (NULL, 0);
-
-    g_debug("%s: select uris", gdk_atom_name (gtk_clipboard_get_selection (priv->real)));
-
-    g_paste_clipboard_private_set_text (priv, g_paste_item_get_real_value (_G_PASTE_ITEM (item)));
-
-    gtk_target_list_add_text_targets (target_list, 0);
-    gtk_target_list_add_uri_targets (target_list, 0);
-    gtk_target_list_add (target_list, g_paste_clipboard_copy_files_target, 0, 0);
-
-    gint32 n_targets;
-    GtkTargetEntry *targets = gtk_target_table_new_from_list (target_list, &n_targets);
-    gtk_clipboard_set_with_owner (real,
-                                  targets,
-                                  n_targets,
-                                  g_paste_clipboard_get_clipboard_data,
-                                  g_paste_clipboard_clear_clipboard_data,
-                                  g_object_ref (item));
-    gtk_target_table_free (targets, n_targets);
-}
-
 /**
  * g_paste_clipboard_clear:
  * @self: a #GPasteClipboard instance
@@ -380,7 +433,7 @@ g_paste_clipboard_clear (GPasteClipboard *self)
     if (!priv->text && !priv->image_checksum)
         return;
 
-    g_debug("%s: clear", gdk_atom_name (gtk_clipboard_get_selection (priv->real)));
+    g_debug("%s: clear", _g_paste_clipboard_private_target_name (priv));
 
     g_clear_pointer (&priv->text, g_free);
     g_clear_pointer (&priv->image_checksum, g_free);
@@ -401,7 +454,7 @@ g_paste_clipboard_store (GPasteClipboard *self)
 
     GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
 
-    g_debug("%s: store", gdk_atom_name (gtk_clipboard_get_selection (priv->real)));
+    g_debug("%s: store", _g_paste_clipboard_private_target_name (priv));
 
     gtk_clipboard_store (priv->real);
 }
@@ -444,7 +497,7 @@ g_paste_clipboard_private_select_image (GPasteClipboardPrivate *priv,
 
     GtkClipboard *real = priv->real;
 
-    g_debug("%s: select image", gdk_atom_name (gtk_clipboard_get_selection (priv->real)));
+    g_debug("%s: select image", _g_paste_clipboard_private_target_name (priv));
 
     g_paste_clipboard_private_set_image_checksum (priv, checksum);
     gtk_clipboard_set_image (real, image);
@@ -472,21 +525,12 @@ g_paste_clipboard_on_image_ready (GtkClipboard *clipboard G_GNUC_UNUSED,
     }
 
     GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+    g_autofree gchar *checksum = g_paste_util_compute_checksum (image);
 
-    g_autofree gchar *checksum = g_compute_checksum_for_data (G_CHECKSUM_SHA256,
-                                                                        (guchar *) gdk_pixbuf_get_pixels (image),
-                                                                        -1);
-
-    if (!g_paste_str_equal (checksum, priv->image_checksum))
-    {
-        g_paste_clipboard_private_select_image (priv,
-                                                image,
-                                                checksum);
-    }
-    else
-    {
+    if (g_paste_str_equal (checksum, priv->image_checksum))
         image = NULL;
-    }
+    else
+        g_paste_clipboard_private_select_image (priv, image, checksum);
 
     if (data->callback)
         data->callback (self, image, data->user_data);
@@ -525,44 +569,52 @@ g_paste_clipboard_set_image (GPasteClipboard             *self,
  * @item: the item to select
  *
  * Put the value of the item into the #GPasteClipbaord and the intern GtkClipboard
+ *
+ * Returns: %FALSE if the item was invalid, %TRUE otherwise
  */
-G_PASTE_VISIBLE void
+G_PASTE_VISIBLE gboolean
 g_paste_clipboard_select_item (GPasteClipboard *self,
                                GPasteItem      *item)
 {
-    g_return_if_fail (_G_PASTE_IS_CLIPBOARD (self));
-    g_return_if_fail (_G_PASTE_IS_ITEM (item));
+    g_return_val_if_fail (_G_PASTE_IS_CLIPBOARD (self), FALSE);
+    g_return_val_if_fail (_G_PASTE_IS_ITEM (item), FALSE);
 
     GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+    GtkClipboard *real = priv->real;
+    g_autoptr (GtkTargetList) target_list = gtk_target_list_new (NULL, 0);
 
-    g_debug("%s: select item", gdk_atom_name (gtk_clipboard_get_selection (priv->real)));
+    g_debug("%s: select item", _g_paste_clipboard_private_target_name (priv));
 
     if (_G_PASTE_IS_IMAGE_ITEM (item))
     {
-        const GPasteImageItem *image_item = _G_PASTE_IMAGE_ITEM (item);
-        const gchar *checksum = g_paste_image_item_get_checksum (image_item);
-
-        if (!g_paste_str_equal (checksum, priv->image_checksum))
-        {
-            g_paste_clipboard_private_select_image (priv,
-                                                    g_paste_image_item_get_image (image_item),
-                                                    checksum);
-        }
+        gtk_target_list_add_image_targets (target_list, 0, FALSE);
+        g_paste_clipboard_private_set_image_checksum (priv, g_paste_image_item_get_checksum (G_PASTE_IMAGE_ITEM (item)));
     }
     else
     {
-        const gchar *text = g_paste_item_get_real_value (item);
-
-        if (!g_paste_str_equal (text, priv->text))
-        {
-            if (_G_PASTE_IS_URIS_ITEM (item))
-                g_paste_clipboard_private_select_uris (priv, G_PASTE_URIS_ITEM (item));
-            else  if (_G_PASTE_IS_TEXT_ITEM (item))
-                g_paste_clipboard_select_text (self, text);
-            else
-                g_assert_not_reached ();
-        }
+        gtk_target_list_add_text_targets (target_list, 0);
+        if (_G_PASTE_IS_URIS_ITEM (item))
+            gtk_target_list_add_uri_targets (target_list, 0);
+        g_paste_clipboard_private_set_text (priv, g_paste_item_get_real_value (item));
     }
+
+    for (const GSList *sv = g_paste_item_get_special_values (item); sv; sv = sv->next)
+    {
+        const GPasteSpecialValue *v = sv->data;
+        gtk_target_list_add (target_list, g_paste_special_atom_get (v->mime), 0, 0);
+    }
+
+    gint32 n_targets;
+    GtkTargetEntry *targets = gtk_target_table_new_from_list (target_list, &n_targets);
+    gtk_clipboard_set_with_owner (real,
+                                  targets,
+                                  n_targets,
+                                  g_paste_clipboard_get_clipboard_data,
+                                  g_paste_clipboard_clear_clipboard_data,
+                                  g_object_ref ((GObject *) item));
+    gtk_target_table_free (targets, n_targets);
+
+    return TRUE;
 }
 
 /**
@@ -573,8 +625,8 @@ g_paste_clipboard_select_item (GPasteClipboard *self,
  * Ensure the clipboard has some contents (as long as the history's not empty)
  */
 G_PASTE_VISIBLE void
-g_paste_clipboard_ensure_not_empty (GPasteClipboard     *self,
-                                    const GPasteHistory *history)
+g_paste_clipboard_ensure_not_empty (GPasteClipboard *self,
+                                    GPasteHistory   *history)
 {
     g_return_if_fail (_G_PASTE_IS_CLIPBOARD (self));
     g_return_if_fail (_G_PASTE_IS_HISTORY (history));
@@ -582,7 +634,12 @@ g_paste_clipboard_ensure_not_empty (GPasteClipboard     *self,
     const GList *hist = g_paste_history_get_history (history);
 
     if (hist)
-        g_paste_clipboard_select_item (self, hist->data);
+    {
+        GPasteItem *item = hist->data;
+
+        if (!g_paste_clipboard_select_item (self, item))
+            g_paste_history_remove (history, 0);
+    }
 }
 
 static void
@@ -593,7 +650,7 @@ g_paste_clipboard_owner_change (GtkClipboard        *clipboard G_GNUC_UNUSED,
     GPasteClipboard *self = user_data;
     GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
 
-    g_debug("%s: owner change", gdk_atom_name (gtk_clipboard_get_selection (priv->real)));
+    g_debug("%s: owner change", _g_paste_clipboard_private_target_name (priv));
 
     g_signal_emit (self,
 		   signals[OWNER_CHANGE],
@@ -704,22 +761,10 @@ g_paste_clipboard_init (GPasteClipboard *self G_GNUC_UNUSED)
 {
 }
 
-/**
- * g_paste_clipboard_new:
- * @target: the GdkAtom representating the GtkClipboard we're abstracting
- * @settings: a #GPasteSettings instance
- *
- * Create a new instance of #GPasteClipboard
- *
- * Returns: a newly allocated #GPasteClipboard
- *          free it with g_object_unref
- */
-G_PASTE_VISIBLE GPasteClipboard *
-g_paste_clipboard_new (GdkAtom         target,
-                       GPasteSettings *settings)
+static GPasteClipboard *
+_g_paste_clipboard_new (GPasteSettings *settings,
+                        GdkAtom         target)
 {
-    g_return_val_if_fail (_G_PASTE_IS_SETTINGS (settings), NULL);
-
     GPasteClipboard *self = g_object_new (G_PASTE_TYPE_CLIPBOARD, NULL);
     GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
     GtkClipboard *real = priv->real = gtk_clipboard_get (target);
@@ -738,3 +783,37 @@ g_paste_clipboard_new (GdkAtom         target,
 
     return self;
 }
+
+/**
+ * g_paste_clipboard_new_clipboard:
+ * @settings: a #GPasteSettings instance
+ *
+ * Create a new instance of #GPasteClipboard
+ *
+ * Returns: a newly allocated #GPasteClipboard
+ *          free it with g_object_unref
+ */
+G_PASTE_VISIBLE GPasteClipboard *
+g_paste_clipboard_new_clipboard (GPasteSettings *settings)
+{
+    g_return_val_if_fail (_G_PASTE_IS_SETTINGS (settings), NULL);
+
+    return _g_paste_clipboard_new (settings, GDK_SELECTION_CLIPBOARD);
+}
+
+/**
+ * g_paste_clipboard_new_primary:
+ * @settings: a #GPasteSettings instance
+ *
+ * Create a new instance of #GPasteClipboard
+ *
+ * Returns: a newly allocated #GPasteClipboard
+ *          free it with g_object_unref
+ */
+G_PASTE_VISIBLE GPasteClipboard *
+g_paste_clipboard_new_primary (GPasteSettings *settings)
+{
+    g_return_val_if_fail (_G_PASTE_IS_SETTINGS (settings), NULL);
+
+    return _g_paste_clipboard_new (settings, GDK_SELECTION_PRIMARY);
+}
