@@ -1,7 +1,7 @@
 /*
  * This file is part of GPaste.
  *
- * Copyright (c) 2010-2016, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
+ * Copyright (c) 2010-2018, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  */
 
 #include "gpaste-gdbus-macros.h"
@@ -678,11 +678,15 @@ g_paste_daemon_private_search (const GPasteDaemonPrivate *priv,
 }
 
 static void
-g_paste_daemon_private_select (const GPasteDaemonPrivate *priv,
-                               GVariant                  *parameters)
+g_paste_daemon_select (const GPasteDaemon *self,
+                       GVariant           *parameters)
 {
-    g_paste_history_select (priv->history,
-                            g_paste_daemon_get_dbus_uint64_parameter (parameters));
+    const GPasteDaemonPrivate *priv = _g_paste_daemon_get_instance_private (self);
+    guint64 position = g_paste_daemon_get_dbus_uint64_parameter (parameters);
+
+    g_paste_history_select (priv->history, position);
+
+    G_PASTE_SEND_DBUS_SIGNAL_FULL (ITEM_SELECTED, g_variant_new_uint64 (position), NULL);
 }
 
 static void
@@ -739,6 +743,7 @@ g_paste_daemon_private_set_password (const GPasteDaemonPrivate *priv,
     g_autofree gchar *name = g_variant_dup_string (variant2, &length);
 
     G_PASTE_DBUS_ASSERT (name, "no password name given");
+    G_PASTE_DBUS_ASSERT (!g_paste_history_get_password (priv->history, name), "a password with tat name already exists");
 
     g_paste_history_set_password (priv->history, index, name);
 }
@@ -894,7 +899,7 @@ g_paste_daemon_dbus_method_call (GDBusConnection       *connection     G_GNUC_UN
     else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_SEARCH))
         answer = g_paste_daemon_private_search (priv, parameters, &err);
     else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_SELECT))
-        g_paste_daemon_private_select (priv, parameters);
+        g_paste_daemon_select (self, parameters);
     else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_SET_PASSWORD))
         g_paste_daemon_private_set_password (priv, parameters, &err);
     else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_SHOW_HISTORY))
@@ -1142,8 +1147,8 @@ g_paste_daemon_init (GPasteDaemon *self)
     GPasteHistory *history = priv->history = g_paste_history_new (settings);
     GPasteClipboardsManager *clipboards_manager = priv->clipboards_manager = g_paste_clipboards_manager_new (history, settings);
 
-    g_autoptr (GPasteClipboard) clipboard = g_paste_clipboard_new (GDK_SELECTION_CLIPBOARD, settings);
-    g_autoptr (GPasteClipboard) primary = g_paste_clipboard_new (GDK_SELECTION_PRIMARY, settings);
+    g_autoptr (GPasteClipboard) clipboard = g_paste_clipboard_new_clipboard (settings);
+    g_autoptr (GPasteClipboard) primary = g_paste_clipboard_new_primary (settings);
 
     g_paste_clipboards_manager_add_clipboard (clipboards_manager, clipboard);
     g_paste_clipboards_manager_add_clipboard (clipboards_manager, primary);
