@@ -1,11 +1,10 @@
 /*
  * This file is part of GPaste.
  *
- * Copyright (c) 2010-2016, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
+ * Copyright (c) 2010-2018, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  */
 
 #include <gpaste-gsettings-keys.h>
-#include <gpaste-settings.h>
 #include <gpaste-util.h>
 
 struct _GPasteSettings
@@ -26,7 +25,9 @@ typedef struct
     GSettings *settings;
     GSettings *shell_settings;
 
+    gboolean   close_on_select;
     guint64    element_size;
+    gboolean   empty_history_confirmation;
     gboolean   growing_lines;
     gchar     *history_name;
     gboolean   images_support;
@@ -39,6 +40,7 @@ typedef struct
     guint64    min_text_item_size;
     gchar     *pop;
     gboolean   primary_to_history;
+    gboolean   rich_text_support;
     gboolean   save_history;
     gchar     *show_history;
     gchar     *sync_clipboard_to_primary;
@@ -103,7 +105,7 @@ static guint64 signals[LAST_SIGNAL] = { 0 };
     SETTING (name, key, type, setting_type, fail, {}, {},)
 
 #define BOOLEAN_SETTING(name, key) TRIVIAL_SETTING (name, key, gboolean, boolean, FALSE)
-#define UNSIGNED_SETTING(name, key) TRIVIAL_SETTING (name, key, guint64, gpaste_uint64, 0)
+#define UNSIGNED_SETTING(name, key) TRIVIAL_SETTING (name, key, guint64, uint64, 0)
 
 #define STRING_SETTING(name, key) SETTING (name, key, const gchar *, string, NULL,                \
                                            g_return_if_fail (value);                              \
@@ -125,22 +127,28 @@ static guint64 signals[LAST_SIGNAL] = { 0 };
 #define NEW_SIGNAL_DETAILED(name, arg_type) NEW_SIGNAL_FULL (name, G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED, arg_type, arg_type)
 #define NEW_SIGNAL_DETAILED_STATIC(name, arg_type) NEW_SIGNAL_FULL (name, G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED, arg_type, arg_type | G_SIGNAL_TYPE_STATIC_SCOPE)
 
-static guint64
-g_settings_get_gpaste_uint64 (GSettings   *settings,
-                              const gchar *key)
-{
-    g_autoptr (GVariant) v = g_settings_get_value (settings, key);
-
-    return g_variant_get_uint64 (v);
-}
-
-static void
-g_settings_set_gpaste_uint64 (GSettings   *settings,
-                              const gchar *key,
-                              guint64      value)
-{
-    g_settings_set_value (settings, key, g_variant_new_uint64 (value));
-}
+/**
+ * g_paste_settings_get_close_on_select:
+ * @self: a #GPasteSettings instance
+ *
+ * Get the "close-on-select" setting
+ *
+ * Returns: the value of the "close-on-select" setting
+ */
+/**
+ * g_paste_settings_reset_close_on_select:
+ * @self: a #GPasteSettings instance
+ *
+ * Reset the "close-on-select" setting
+ */
+/**
+ * g_paste_settings_set_close_on_select:
+ * @self: a #GPasteSettings instance
+ * @value: the new history name
+ *
+ * Change the "close-on-select" setting
+ */
+BOOLEAN_SETTING (close_on_select, CLOSE_ON_SELECT)
 
 /**
  * g_paste_settings_get_element_size:
@@ -165,6 +173,29 @@ g_settings_set_gpaste_uint64 (GSettings   *settings,
  */
 UNSIGNED_SETTING (element_size, ELEMENT_SIZE)
 
+/**
+ * g_paste_settings_get_empty_history_confirmation:
+ * @self: a #GPasteSettings instance
+ *
+ * Get the "empty-history-confirmation" setting
+ *
+ * Returns: the value of the "empty-history-confirmation" setting
+ */
+/**
+ * g_paste_settings_reset_empty_history_confirmation:
+ * @self: a #GPasteSettings instance
+ *
+ * Reset the "empty-history-confirmation" setting
+ */
+/**
+ * g_paste_settings_set_empty_history_confirmation:
+ * @self: a #GPasteSettings instance
+ * @value: whether to prompt for confirmation when emptying an history
+ *
+ * Change the "empty-history-confirmation" setting
+ */
+BOOLEAN_SETTING (empty_history_confirmation, EMPTY_HISTORY_CONFIRMATION)
+
 /**
  * g_paste_settings_get_growing_lines:
  * @self: a #GPasteSettings instance
@@ -441,6 +472,29 @@ STRING_SETTING (pop, POP)
  */
 BOOLEAN_SETTING (primary_to_history, PRIMARY_TO_HISTORY)
 
+/**
+ * g_paste_settings_get_rich_text_support:
+ * @self: a #GPasteSettings instance
+ *
+ * Get the "rich-text-support" setting
+ *
+ * Returns: the value of the "rich-text-support" setting
+ */
+/**
+ * g_paste_settings_reset_rich_text_support:
+ * @self: a #GPasteSettings instance
+ *
+ * Reset the "rich-text-support" setting
+ */
+/**
+ * g_paste_settings_set_rich_text_support:
+ * @self: a #GPasteSettings instance
+ * @value: the new history name
+ *
+ * Change the "rich-text-support" setting
+ */
+BOOLEAN_SETTING (rich_text_support, RICH_TEXT_SUPPORT)
+
 /**
  * g_paste_settings_get_save_history:
  * @self: a #GPasteSettings instance
@@ -767,8 +821,12 @@ g_paste_settings_settings_changed (GSettings   *settings G_GNUC_UNUSED,
     GPasteSettings *self = G_PASTE_SETTINGS (user_data);
     GPasteSettingsPrivate *priv = g_paste_settings_get_instance_private (self);
 
-    if (g_paste_str_equal (key, G_PASTE_ELEMENT_SIZE_SETTING))
+    if (g_paste_str_equal (key, G_PASTE_CLOSE_ON_SELECT_SETTING))
+        g_paste_settings_private_set_close_on_select_from_dconf (priv);
+    else if (g_paste_str_equal (key, G_PASTE_ELEMENT_SIZE_SETTING))
         g_paste_settings_private_set_element_size_from_dconf (priv);
+    else if (g_paste_str_equal (key, G_PASTE_EMPTY_HISTORY_CONFIRMATION_SETTING))
+        g_paste_settings_private_set_empty_history_confirmation_from_dconf (priv);
     else if (g_paste_str_equal (key, G_PASTE_GROWING_LINES_SETTING))
         g_paste_settings_private_set_growing_lines_from_dconf (priv);
     else if (g_paste_str_equal (key, G_PASTE_HISTORY_NAME_SETTING))
@@ -802,6 +860,8 @@ g_paste_settings_settings_changed (GSettings   *settings G_GNUC_UNUSED,
     }
     else if (g_paste_str_equal (key, G_PASTE_PRIMARY_TO_HISTORY_SETTING ))
         g_paste_settings_private_set_primary_to_history_from_dconf (priv);
+    else if (g_paste_str_equal (key, G_PASTE_RICH_TEXT_SUPPORT_SETTING))
+        g_paste_settings_private_set_rich_text_support_from_dconf (priv);
     else if (g_paste_str_equal (key, G_PASTE_SAVE_HISTORY_SETTING))
         g_paste_settings_private_set_save_history_from_dconf (priv);
     else if (g_paste_str_equal (key, G_PASTE_SHOW_HISTORY_SETTING))
@@ -956,7 +1016,9 @@ g_paste_settings_init (GPasteSettings *self)
                                                    G_CALLBACK (g_paste_settings_settings_changed),
                                                    self);
 
+    g_paste_settings_private_set_close_on_select_from_dconf (priv);
     g_paste_settings_private_set_element_size_from_dconf (priv);
+    g_paste_settings_private_set_empty_history_confirmation_from_dconf (priv);
     g_paste_settings_private_set_growing_lines_from_dconf (priv);
     g_paste_settings_private_set_history_name_from_dconf (priv);
     g_paste_settings_private_set_images_support_from_dconf (priv);
@@ -969,6 +1031,7 @@ g_paste_settings_init (GPasteSettings *self)
     g_paste_settings_private_set_min_text_item_size_from_dconf (priv);
     g_paste_settings_private_set_pop_from_dconf (priv);
     g_paste_settings_private_set_primary_to_history_from_dconf (priv);
+    g_paste_settings_private_set_rich_text_support_from_dconf (priv);
     g_paste_settings_private_set_save_history_from_dconf (priv);
     g_paste_settings_private_set_show_history_from_dconf (priv);
     g_paste_settings_private_set_sync_clipboard_to_primary_from_dconf (priv);
