@@ -1,13 +1,14 @@
 /*
  * This file is part of GPaste.
  *
- * Copyright (c) 2010-2016, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
+ * Copyright (c) 2010-2018, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  */
 
 #include <gpaste-gsettings-keys.h>
-#include <gpaste-macros.h>
 #include <gpaste-util.h>
 
+#include "gpaste-gtk-compat.h"
+
 /**
  * g_paste_util_confirm_dialog:
  * @parent: (nullable): the parent #GtkWindow
@@ -31,11 +32,9 @@ g_paste_util_confirm_dialog (GtkWindow   *parent,
     GtkWidget *label = gtk_label_new (msg);
     GtkDialog *d = GTK_DIALOG (dialog);
 
-    gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (d)), label, TRUE, TRUE, 0);
-    gtk_widget_set_margin_start (label, 10);
-    gtk_widget_set_margin_end (label, 10);
-    gtk_widget_set_margin_top (label, 10);
-    gtk_widget_set_margin_bottom (label, 10);
+    gtk_widget_set_vexpand (label, TRUE);
+    gtk_widget_set_valign (label, TRUE);
+    gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (d)), label, TRUE, TRUE);
     gtk_widget_show (label);
 
     gboolean ret = gtk_dialog_run (d) == GTK_RESPONSE_OK;
@@ -316,36 +315,31 @@ g_paste_util_compute_checksum (GdkPixbuf *image)
     if (!image || !GDK_IS_PIXBUF (image))
         return NULL;
 
-    guint32 length;
-    const guchar *data = gdk_pixbuf_get_pixels_with_length (image, &length);
+    const guint8 *data = gdk_pixbuf_read_pixels (image);
+    gsize length = gdk_pixbuf_get_byte_length (image);
 
     return g_compute_checksum_for_data (G_CHECKSUM_SHA256, data, length);
 }
 
 /**
- * g_paste_util_has_applet:
+ * g_paste_util_empty_history:
+ * @parent_window: (nullable): the parent #GtkWindow
+ * @client: a #GPasteClient instance
+ * @settings: a #GPasteSettings instance
+ * @history: the name of the history to empty
  *
- * Check whether gpaste-applet is installed or not
- *
- * Returns: %TRUE if gpaste-applet is installed
+ * Empty history after prompting user for confirmation
  */
-G_PASTE_VISIBLE gboolean
-g_paste_util_has_applet (void)
+G_PASTE_VISIBLE void
+g_paste_util_empty_history (GtkWindow      *parent_window,
+                            GPasteClient   *client,
+                            GPasteSettings *settings,
+                            const gchar    *history)
 {
-    return g_file_test (PKGLIBEXECDIR "/gpaste-applet", G_FILE_TEST_IS_EXECUTABLE);
-}
-
-/**
- * g_paste_util_has_unity:
- *
- * Check whether gpaste-app-indicator is installed or not
- *
- * Returns: %TRUE if gpaste-app-indicator is installed
- */
-G_PASTE_VISIBLE gboolean
-g_paste_util_has_unity (void)
-{
-    return g_file_test (PKGLIBEXECDIR "/gpaste-app-indicator", G_FILE_TEST_IS_EXECUTABLE);
+    if (!g_paste_settings_get_empty_history_confirmation (settings) ||
+        /* Translators: this is the translation for emptying the history */
+        g_paste_util_confirm_dialog (parent_window, _("Empty"), _("Do you really want to empty the history?")))
+            g_paste_client_empty_history (client, history, NULL, NULL);
 }
 
 /**
@@ -381,7 +375,7 @@ g_paste_util_show_win (GApplication *application)
 
     for (GList *wins = gtk_application_get_windows (GTK_APPLICATION (application)); wins; wins = g_list_next (wins))
     {
-        if (gtk_widget_get_realized (wins->data))
+        if (GTK_IS_WIDGET (wins->data) && gtk_widget_get_realized (wins->data))
             gtk_window_present (wins->data);
     }
 }
@@ -435,6 +429,8 @@ g_paste_util_get_dbus_au_result (GVariant *variant,
 static gchar *
 g_paste_util_get_runtime_dir (const gchar *component)
 {
+    g_return_val_if_fail (component, NULL);
+
     return g_strdup_printf ("%s/" PACKAGE_NAME "/%s", g_get_user_runtime_dir (), component);
 }
 
@@ -447,6 +443,8 @@ g_paste_util_get_runtime_dir (const gchar *component)
 G_PASTE_VISIBLE void
 g_paste_util_write_pid_file (const gchar *component)
 {
+    g_return_if_fail (component);
+
     g_autofree gchar *dir = g_paste_util_get_runtime_dir (component);
 
     g_mkdir_with_parents (dir, 0700);
@@ -468,6 +466,8 @@ g_paste_util_write_pid_file (const gchar *component)
 G_PASTE_VISIBLE GPid
 g_paste_util_read_pid_file (const gchar *component)
 {
+    g_return_val_if_fail (component, (GPid) -1);
+
 #ifdef G_OS_UNIX
     g_autofree gchar *dir = g_paste_util_get_runtime_dir (component);
     g_autofree gchar *pidfile = g_strdup_printf ("%s/pid", dir);
@@ -481,3 +481,146 @@ g_paste_util_read_pid_file (const gchar *component)
     return (GPid) -1;
 #endif
 }
+
+/**
+ * g_paste_util_xml_decode:
+ * @text: The text to decode
+ *
+ * Decode the text to its original pre-xml form
+ *
+ * Returns: the decoded text
+ */
+G_PASTE_VISIBLE gchar *
+g_paste_util_xml_decode (const gchar *text)
+{
+    g_return_val_if_fail (text, NULL);
+
+    g_autofree gchar *_decoded_text = g_paste_util_replace (text, "&gt;", ">");
+
+    return g_paste_util_replace (_decoded_text, "&amp;", "&");
+}
+
+/**
+ * g_paste_util_xml_encode:
+ * @text: The text to encode
+ *
+ * Encode the text into its xml form
+ *
+ * Returns: the encoded text
+ */
+G_PASTE_VISIBLE gchar *
+g_paste_util_xml_encode (const gchar *text)
+{
+    g_return_val_if_fail (text, NULL);
+
+    g_autofree gchar *_encoded_text = g_paste_util_replace (text, "&", "&amp;");
+
+    return g_paste_util_replace (_encoded_text, ">", "&gt;");
+}
+
+/**
+ * g_paste_util_get_history_dir_path:
+ *
+ * Get the path to the directory where we store the history
+ *
+ * Returns: the directory path
+ */
+G_PASTE_VISIBLE gchar *
+g_paste_util_get_history_dir_path (void)
+{
+    return g_build_filename (g_get_user_data_dir (), PACKAGE, NULL);
+}
+
+/**
+ * g_paste_util_get_history_dir:
+ *
+ * Get the directory where we store the history
+ *
+ * Returns: (transfer full): the directory
+ */
+G_PASTE_VISIBLE GFile *
+g_paste_util_get_history_dir (void)
+{
+    g_autofree gchar *history_dir_path = g_paste_util_get_history_dir_path ();
+
+    return g_file_new_for_path (history_dir_path);
+}
+
+/**
+ * g_paste_util_get_history_file_path:
+ * @name: the name of the history
+ * @extension: the file extension
+ *
+ * Get the path to the file in which we store the history
+ *
+ * Returns: the file path
+ */
+G_PASTE_VISIBLE gchar *
+g_paste_util_get_history_file_path (const gchar *name,
+                                    const gchar *extension)
+{
+    g_return_val_if_fail (name, NULL);
+    g_return_val_if_fail (extension, NULL);
+
+    g_autofree gchar *history_dir_path = g_paste_util_get_history_dir_path ();
+    g_autofree gchar *history_file_name = g_strconcat (name, ".", extension, NULL);
+
+    return g_build_filename (history_dir_path, history_file_name, NULL);
+}
+
+/**
+ * g_paste_util_get_history_file:
+ * @name: the name of the history
+ * @extension: the file extension
+ *
+ * Get the file in which we store the history
+ *
+ * Returns: (transfer full): the file
+ */
+G_PASTE_VISIBLE GFile *
+g_paste_util_get_history_file (const gchar *name,
+                               const gchar *extension)
+{
+    g_return_val_if_fail (name, NULL);
+    g_return_val_if_fail (extension, NULL);
+
+    g_autofree gchar *history_file_path = g_paste_util_get_history_file_path (name, extension);
+
+    return g_file_new_for_path (history_file_path);
+}
+
+/**
+ * g_paste_util_ensure_history_dir_exists:
+ * @settings: a #GPasteSettings instance
+ *
+ * Ensure the history dir exists
+ *
+ * Returns: where it exists or if there was an error creating it
+ */
+G_PASTE_VISIBLE gboolean
+g_paste_util_ensure_history_dir_exists (const GPasteSettings *settings)
+{
+    g_return_val_if_fail (_G_PASTE_IS_SETTINGS (settings), FALSE);
+
+    g_autoptr (GFile) history_dir = g_paste_util_get_history_dir ();
+
+    if (!g_file_query_exists (history_dir,
+                              NULL)) /* cancellable */
+    {
+        if (!g_paste_settings_get_save_history (settings))
+            return TRUE;
+
+        g_autoptr (GError) error = NULL;
+
+        g_file_make_directory_with_parents (history_dir,
+                                            NULL, /* cancellable */
+                                            &error);
+        if (error)
+        {
+            g_critical ("%s: %s", _("Could not create history dir"), error->message);
+            return FALSE;
+        }
+    }
+
+    return TRUE;
+}
